@@ -104,6 +104,54 @@ pub enum RequestInner<'a> {
     /// Responds with ResponseInner::Name with a name and dummy attribute value
     /// or ResponseInner::Status on error.
     Realpath(Cow<'a, Path>),
+
+    /// Responds with extended reply, with payload `response::Limits`.
+    ///
+    /// Extension, only available if it is `Extensions::limits`
+    /// is returned by `response::HelloVersion`
+    Limits,
+
+    /// Same response as `ResponseInner::Realpath`.
+    ///
+    /// Extension, only available if it is `Extensions::expand_path`
+    /// is returned by `response::HelloVersion`.
+    ///
+    /// This supports canonicalisation of relative paths and those that need
+    /// tilde-expansion, i.e. "~", "~/..." and "~user/...".
+    ///
+    /// These paths are expanded using shell-lilke rules and the resultant path
+    /// is canonicalised similarly to `RequestInner::Realpath`.
+    ExpandPath(Cow<'a, Path>),
+
+    /// Same response as `ResponseInner::Setstat`.
+    ///
+    /// Extension, only available if it is `Extensions::lsetstat`
+    /// is returned by `response::HelloVersion`.
+    Lsetstat(Cow<'a, Path>, FileAttrs),
+
+    /// Responds with `ResponseInner::Status`.
+    ///
+    /// Extension, only available if it is `Extensions::fsync`
+    /// is returned by `response::HelloVersion`
+    Fsync(Cow<'a, Handle>),
+
+    /// Responds with `ResponseInner::Status`.
+    ///
+    /// Extension, only available if it is `Extensions::hardlink`
+    /// is returned by `response::HelloVersion`
+    HardLink {
+        oldpath: Cow<'a, Path>,
+        newpath: Cow<'a, Path>,
+    },
+
+    /// Responds with `ResponseInner::Status`.
+    ///
+    /// Extension, only available if it is `Extensions::posix_rename`
+    /// is returned by `response::HelloVersion`
+    PosixRename {
+        oldpath: Cow<'a, Path>,
+        newpath: Cow<'a, Path>,
+    },
 }
 
 #[derive(Debug)]
@@ -166,10 +214,60 @@ impl Serialize for Request<'_> {
                 linkpath,
                 targetpath,
             } => {
-                (constants::SSH_FXP_SYMLINK, request_id, linkpath, targetpath).serialize(serializer)
+                (constants::SSH_FXP_SYMLINK, request_id, targetpath, linkpath).serialize(serializer)
             }
 
             Realpath(path) => (constants::SSH_FXP_REALPATH, request_id, path).serialize(serializer),
+
+            Limits => (
+                constants::SSH_FXP_EXTENDED,
+                request_id,
+                constants::EXT_NAME_LIMITS.0,
+            )
+                .serialize(serializer),
+
+            ExpandPath(path) => (
+                constants::SSH_FXP_EXTENDED,
+                request_id,
+                constants::EXT_NAME_EXPAND_PATH.0,
+                path,
+            )
+                .serialize(serializer),
+
+            Lsetstat(path, attrs) => (
+                constants::SSH_FXP_EXTENDED,
+                request_id,
+                constants::EXT_NAME_LSETSTAT.0,
+                path,
+                attrs,
+            )
+                .serialize(serializer),
+
+            Fsync(handle) => (
+                constants::SSH_FXP_EXTENDED,
+                request_id,
+                constants::EXT_NAME_FSYNC.0,
+                handle,
+            )
+                .serialize(serializer),
+
+            HardLink { oldpath, newpath } => (
+                constants::SSH_FXP_EXTENDED,
+                request_id,
+                constants::EXT_NAME_HARDLINK.0,
+                oldpath,
+                newpath,
+            )
+                .serialize(serializer),
+
+            PosixRename { oldpath, newpath } => (
+                constants::SSH_FXP_EXTENDED,
+                request_id,
+                constants::EXT_NAME_POSIX_RENAME.0,
+                oldpath,
+                newpath,
+            )
+                .serialize(serializer),
         }
     }
 }
