@@ -102,6 +102,16 @@ impl ServerVersion {
     }
 }
 
+/// Payload of extended reply response when `ResponseInner::Limits`
+/// is sent.
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Deserialize)]
+pub struct Limits {
+    packet_len: u64,
+    read_len: u64,
+    write_len: u64,
+    open_handles: u64,
+}
+
 #[derive(Debug)]
 pub enum ResponseInner {
     Status {
@@ -112,8 +122,6 @@ pub enum ResponseInner {
 
     Handle(HandleOwned),
 
-    Data,
-
     Name(Box<[NameEntry]>),
 
     Attrs(FileAttrsBox),
@@ -126,12 +134,15 @@ pub struct Response {
 }
 
 impl Response {
-    /// Return Some(true) if the packet is data,
-    /// Some(false) if not,
-    /// None if it isn't a valid response.
+    /// Return true if the response is a data response.
     pub fn is_data(packet_type: u8) -> bool {
         packet_type == constants::SSH_FXP_DATA
     }
+
+    /// Return true if the response is a extended reply response.
+    pub fn is_extended_reply(packet_type: u8) -> bool {
+        packet_type == constants::SSH_FXP_EXTENDED_REPLY
+    }
 }
 
 impl_visitor!(
@@ -156,15 +167,17 @@ impl_visitor!(
 
             SSH_FXP_HANDLE => Handle(iter.get_next()?),
 
-            SSH_FXP_DATA => Data,
-
             SSH_FXP_NAME => {
                 let len: u32 = iter.get_next()?;
                 let len = len as usize;
                 let mut entries = Vec::<NameEntry>::with_capacity(len);
 
                 for _ in 0..len {
-                    entries.push(iter.get_next()?);
+                    let filename: Box<Path> = iter.get_next()?;
+                    let _longname: &[u8] = iter.get_next()?;
+                    let attrs: FileAttrs = iter.get_next()?;
+
+                    entries.push(NameEntry { filename, attrs });
                 }
 
                 Name(entries.into_boxed_slice())
@@ -189,13 +202,6 @@ impl_visitor!(
 
 #[derive(Debug, Copy, Clone)]
 pub enum ErrorCode {
-    /// Indicates end-of-file condition.
-    ///
-    /// For RequestInner::Read it means that no more data is available in the file,
-    /// and for RequestInner::Readdir it indicates that no more files are contained
-    /// in the directory.
-    Eof,
-
     /// is returned when a reference is made to a file which should exist
     /// but doesn't.
     NoSuchFile,
@@ -212,6 +218,9 @@ pub enum ErrorCode {
 
     /// May be returned if a badly formatted packet or protocol
     /// incompatibility is detected.
+    ///
+    /// If the handle is opened read only, but write flag is required,
+    /// then `BadMessage` might be returned, vice versa.
     BadMessage,
 
     /// Indicates that an attempt was made to perform an operation which
@@ -223,6 +232,13 @@ pub enum ErrorCode {
 pub enum StatusCode {
     Success,
     Failure(ErrorCode),
+
+    /// Indicates end-of-file condition.
+    ///
+    /// For RequestInner::Read it means that no more data is available in the file,
+    /// and for RequestInner::Readdir it indicates that no more files are contained
+    /// in the directory.
+    Eof,
 }
 impl<'de> Deserialize<'de> for StatusCode {
     fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
@@ -233,7 +249,7 @@ impl<'de> Deserialize<'de> for StatusCode {
 
         match discriminant {
             SSH_FX_OK => Ok(StatusCode::Success),
-            SSH_FX_EOF => Ok(StatusCode::Failure(Eof)),
+            SSH_FX_EOF => Ok(StatusCode::Eof),
             SSH_FX_NO_SUCH_FILE => Ok(StatusCode::Failure(NoSuchFile)),
             SSH_FX_PERMISSION_DENIED => Ok(StatusCode::Failure(PermDenied)),
             SSH_FX_FAILURE => Ok(StatusCode::Failure(Failure)),
@@ -289,23 +305,9 @@ impl fmt::Debug for ErrMsg {
 }
 
 /// Entry in ResponseInner::Name
-#[derive(Debug, Deserialize, Clone)]
+#[derive(Debug, Clone)]
 pub struct NameEntry {
     pub filename: Box<Path>,
 
-    /// The format of the `longname' field is unspecified by this protocol.
-    ///
-    /// It MUST be suitable for use in the output of a directory listing
-    /// command (in fact, the recommended operation for a directory listing
-    /// command is to simply display this data).
-    ///
-    /// However, clients SHOULD NOT attempt to parse the longname field for file
-    /// attributes, they SHOULD use the attrs field instead.
-    ///
-    /// The recommended format for the longname field is as follows:
-    ///
-    /// -rwxr-xr-x   1 mjos     staff      348911 Mar 25 14:29 t-filexfer
-    /// 1234567890 123 12345678 12345678 12345678 123456789012
-    pub longname: Box<str>,
     pub attrs: FileAttrs,
 }
